import numpy as np
import matplotlib.pyplot as plt
from scipy.integrate import solve_ivp
from HtCapFn import HtCap
from PBRsimplifiedFn import PBRsimplified
# basic data
Rgas = 8.314  # kJ/kmol/K
# molecular weights
MWN2 = 28.0134  # kg/kmol
MWH2 = 2.016
MWNH3 = 17.031
MWAr = 39.948
MWCH4 = 16.043
# heat capacity coefficients
# from fit of Hysys properties
# at 150 atm
CpCoef = np.array([[40.442,  -35.279,   46.918,  -19.41,  0],
                  [28.75,      1.86,     0,        0,     0],
                  [1088.5, -5691.4,  11787,   -10900,  3801.6],
                  [36.819,  -53.01,     63.973,  -27.022, 0],
                  [24.709,   50.297,     0,        0,     0]])
# heat capacities at 350 degC
Tmid = 350  # degC
CpN2 = HtCap(1,Tmid,CpCoef)
CpH2 = HtCap(2,Tmid,CpCoef)
CpNH3 = HtCap(3,Tmid,CpCoef)
CpAr = HtCap(4,Tmid,CpCoef)
CpCH4 = HtCap(5,Tmid,CpCoef)                  
# reaction kinetics
#forward reaction
k0f = 3.6e7  # kmol/m3/h/atm
Ef = 9.1e4  # kJ/kmol
# reverse reaction
k0r = 4.68e13  # kmol/m3/h/atm
Er = 1.41e5  # kJ/kmol
# reactor parameters
Dr = 3  # diameter, m
Lr = 1  # length, m
Ar = np.pi*Dr**2/4  # x-sectional area, m2
Vr = Ar*Lr  # volume, m3
# catalyst particles and packing
Dp = 1.e-3  # particle diameter, m
eps = 0.4  # void fraction
# feed conditions
FeedN2 = 12348  # kmol/h
FeedH2 = 37044  
FeedNH3 = 0
FeedAr = 12391
FeedCH4 = 5652
FeedP = 150  # atm
FeedT = 270  # degC
# initial conditions
y0 = np.array([FeedN2, FeedT])
# solution span and intervals
vspan = [0., Vr]
veval = np.linspace(0,Vr,200)
# solve model
result = solve_ivp(PBRsimplified,vspan,y0,t_eval=veval,
                         args=(FeedN2,FeedH2,FeedAr,FeedCH4,
                         FeedP,k0f,Ef,k0r,Er,eps,
                         CpN2,CpH2,CpNH3,CpAr,CpCH4))
vs = result.t
MflowN2 = result.y[0,:]
MflowH2 = FeedH2 - 3*(FeedN2 - MflowN2)
MflowNH3 = 2*(FeedN2 - MflowN2)
T = result.y[1,:]
# conversion, N2 basis, in %
Conv = (FeedN2 - MflowN2)/FeedN2 * 100
# create plots
# temperature profile
plt.figure()  
plt.plot(vs,T,c='k')
plt.grid()
plt.xlabel('Reactor Volume - m3')
plt.ylabel('Temperature - degC')
plt.title('Temperature Profile')
# molar flows profiles
plt.figure()
plt.plot(vs,MflowN2,c='b',label='N2')
plt.plot(vs,MflowH2,'m',label='H2')
plt.plot(vs,MflowNH3,'g',label='NH3')
plt.grid()
plt.xlabel('Reactor Volume - m3')
plt.ylabel('Molar Flow Rate - kmol/h')
plt.title('Molar Flow Profiles')
plt.legend()
# conversion profile
plt.figure()
plt.plot(vs,Conv,c='k')
plt.grid()
plt.xlabel('Reactor Volume - m3')
plt.ylabel('Conversion - %')
plt.title('Conversion Profile')
# print exit conditions
n = len(vs)
print('Exit conversion = {0:4.1f} %'.format(Conv[n-1]))
print('Exit temperature = {0:6.1f} degC'.format(T[n-1]))


