import numpy as np
import matplotlib.pyplot as plt
from scipy.integrate import solve_ivp
from HtCapFn import HtCap
from PBRFullFn import PBRFull
from FindTFn import findT
# basic data
Rgas = 8.314  # kJ/kmol/K
# molecular weights
MWN2 = 28.0134  # kg/kmol
MWH2 = 2.016
MWNH3 = 17.031
MWAr = 39.948
MWCH4 = 16.043
# heat capacity coefficients
# from fit of Hysys properties
# at 150 atm
CpCoef = np.array([[40.442,  -35.279,   46.918,  -19.41,  0],
                  [28.75,      1.86,     0,        0,     0],
                  [1088.5, -5691.4,  11787,   -10900,  3801.6],
                  [36.819,  -53.01,     63.973,  -27.022, 0],
                  [24.709,   50.297,     0,        0,     0]])
# reaction kinetics
#forward reaction
k0f = 3.6e7  # kmol/m3/h/atm
Ef = 9.1e4  # kJ/kmol
# reverse reaction
k0r = 4.68e13  # kmol/m3/h/atm
Er = 1.41e5  # kJ/kmol
# reactor parameters
Dr = 3  # diameter, m
Lr = 1  # length, m
Ar = np.pi*Dr**2/4  # x-sectional area, m2
Vr = Ar*Lr  # volume, m3
# catalyst particles and packing
Dp = 1.e-3  # particle diameter, m
eps = 0.4  # void fraction
# gas kinematic viscosity
nu = 5.075e-7  # m2/s
# feed conditions
FeedN2 = 12348  # kmol/h
FeedH2 = 37044  
FeedNH3 = 0
FeedAr = 12391
FeedCH4 = 5652
FeedP = 150  # atm
FeedT = 270  # degC
# mass flux
M0 = FeedN2*MWN2+FeedH2*MWH2+FeedNH3*MWNH3+FeedAr*MWAr+FeedCH4*MWCH4
G0 = M0/Ar/3600  # kg/s/m2
# initial conditions
T0 = FeedT
H0 = (FeedN2*HtCap(1,T0,CpCoef)+FeedH2*HtCap(2,T0,CpCoef) \
      +FeedNH3*HtCap(3,T0,CpCoef)+FeedAr*HtCap(4,T0,CpCoef) \
      +FeedCH4+HtCap(5,T0,CpCoef))*T0
y0 = np.array([FeedN2, FeedP, H0])
# solution span and intervals
vspan = [0., Vr]
veval = np.linspace(0,Vr,200)
# solve model
result = solve_ivp(PBRFull,vspan,y0,t_eval=veval,
                         args=(FeedN2,FeedH2,FeedAr,FeedCH4,
                         FeedT,
                         k0f,Ef,k0r,Er,eps,nu,G0,Dp,Ar,
                         MWN2,MWH2,MWNH3,MWAr,MWCH4,CpCoef))
vs = result.t
MflowN2 = result.y[0,:]
MflowH2 = FeedH2 - 3*(FeedN2 - MflowN2)
MflowNH3 = 2*(FeedN2 - MflowN2)
P = result.y[1,:]
H = result.y[2,:]
# solve for T profile
n = len(vs)
T = np.zeros(n)
for i in range(n):
    Tmid = 350
    T[i] = findT(H[i],Tmid,MflowN2[i],MflowH2[i],MflowNH3[i] \
                 ,FeedAr,FeedCH4,CpCoef)
# conversion, N2 basis, in %
Conv = (FeedN2 - MflowN2)/FeedN2 * 100
# create plots
# temperature profile
plt.figure()  
plt.plot(vs,T,c='k')
plt.grid()
plt.xlabel('Reactor Volume - m3')
plt.ylabel('Temperature - degC')
plt.title('Temperature Profile')
# pressure profile
plt.figure()
plt.plot(vs,P,c='k')
plt.grid()
plt.xlabel('Reactor Volume - m3')
plt.ylabel('Pressure - atm')
plt.title('Pressure Profile')
# molar flows profiles
plt.figure()
plt.plot(vs,MflowN2,c='b',label='N2')
plt.plot(vs,MflowH2,'m',label='H2')
plt.plot(vs,MflowNH3,'g',label='NH3')
plt.grid()
plt.xlabel('Reactor Volume - m3')
plt.ylabel('Molar Flow Rate - kmol/h')
plt.title('Molar Flow Profiles')
plt.legend()
# conversion profile
plt.figure()
plt.plot(vs,Conv,c='k')
plt.grid()
plt.xlabel('Reactor Volume - m3')
plt.ylabel('Conversion - %')
plt.title('Conversion Profile')
# print exit conditions
n = len(vs)
print('Exit conversion = {0:4.1f} %'.format(Conv[n-1]))
print('Exit temperature = {0:6.1f} degC'.format(T[n-1]))


